/* Copyright (C) 2002 GFRN systems

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
   See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   The latest version of this program may be found at
   http://CQiNet.sourceforge.net

   $Log: configvars.c,v $
   Revision 1.13  2003/09/06 19:05:16  wb6ymh
   Added MaxDups.
   Changed default value for StationListInterval to 600 (10 minutes).

   Revision 1.12  2003/09/01 15:15:01  wb6ymh
   1. Added configuration variables EmailAdr, CmdPort, IdleTimeout, SF_ReplyPort,
      DirCompression, EventScript, StartupCmd, HelpFile and WelcomeDelay.
   2. Modified for multipass initialization, primarily to support StartupCmd.

   Revision 1.11  2003/04/30 21:35:58  wb6ymh
   1. Added configuration varibles Bind2IP, AppName, RunDir, AutoLurkTimeout,
      DefaultAutoLurk, BlabOffTimer, and LogLevel.
   2. Added SetQth() to verify that the default Qth has been edited (sigh).

   Revision 1.10  2002/12/21 18:17:24  wb6ymh
   Added configuration variables MaxPlayWithoutPause, SaveInfoFiles,
   ShowStationInfo and WriteHostFile.

   Revision 1.9  2002/12/18 00:56:59  wb6ymh
   Added WelcomeFile, MaxPlayBackPause, MinPlayBackPause, LogFileRolloverType,
   and AudioTestConf.

   Revision 1.8  2002/11/02 18:15:23  wb6ymh
   1. Added config variables RTP_Pass, PrivateConference, EnableDiskCommands,
      iLinkDirServer, DnsUpdateInterval, BelchTime, InactiveDirTimeout and
      bQuickStart.
   2. Added routine to set ConferenceCall so we can check for an unedited
      config file earlier in the game. (Otherwise a bad working directory
      is reported before the default conference call is detected confusing
      lusers).

   Revision 1.7  2002/10/25 14:27:30  wb6ymh
   Added iLinkDirServer config variable.

   Revision 1.6  2002/09/29 16:32:55  wb6ymh
   Added configuration variables SysopPass and PauseTime.

   Revision 1.5  2002/09/14 16:49:21  wb6ymh
   Added WorkingDir, RunAsUser, Banner, UserCountEnable, MaxCountEnable, and
   UserCountInLocation variables. Deleted BannerEable and ConnectTo variables.

   Revision 1.4  2002/09/01 04:55:40  wb6ymh
   Fixed yet another more pointer to pointer bug.... sigh.

   Revision 1.3  2002/09/01 03:05:16  wb6ymh
   Pointers to pointers make my head hurt!  Fixed bug in SetConfigDefaults().
   GCC V 2.95.3 under FreeBSD didn't issue a warning, but GCC V 2.95.3 on
   Linux and Visual C++ did.  Weird!

   Revision 1.2  2002/08/31 23:42:36  wb6ymh
   Added configuration variables DirServerHost1 ...3, AdminPass, BannerEnable,
   SF_Port, SF_Enable, SF_SWL_Only, LinkConferences. Modified SetConfigDefaults()
   to free all "%s" type variables.  Changed DirServerHost to be an array of
   NUM_DIRECTORY_SERVERS strings.  Changed ConfAudioTimeout default to 800
   milliseconds.

   Revision 1.1.1.1  2002/08/10 20:33:41  wb6ymh
   initial import

*/

#include "common.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#ifndef _WIN32
   #include <unistd.h>
   #include <sys/socket.h>
   #include <netinet/in.h>
   #include <arpa/inet.h>
   #include <sys/types.h>
   #ifdef TIME_WITH_SYS_TIME
      #include <sys/time.h>
      #include <time.h>
   #else
      #ifdef HAVE_SYS_TIME_H
         #include <sys/time.h>
      #else
         #include <time.h>
      #endif
   #endif
   #include <sys/stat.h>
   #define CONFIG_FILE_PATH   SYSCON_DIR "/tbd.conf"
#else
   #include <direct.h>
   #include <time.h>
   #include <sys\timeb.h>
   #include <winsock2.h>
   #include <sys\stat.h>
   #define CONFIG_FILE_PATH   "tbd.conf"
#endif

#include "configfile.h"
#include "configvars.h"
#include "main.h"
#include "conference.h"

// Configuration variables

// String variables: Dynamically allocated
char *ConferenceCall;
char *ConferencePass;
char *ConferenceQth;
char *EmailAdr;
char *ConferenceID;
char *DirServerHost[NUM_DIRECTORY_SERVERS];
char *AdminPass;
char *SysopPass;
char *Banner;
char *WorkingDir;
char *RunAsUser;
char *RTP_Pass;
char *WelcomeFile;
char *Bind2IP;
char *AppName;
char *RunDir;
char *EventScript;
char *CustomHelp;

// Integer variables
int ConfAudioTimeout    = 800;   // in milliseconds (> 8 frames)
int ConfMemberTimeout   = 40;    // in seconds (>= 4 missed frames)
int LoginInterval       = 360;   // in seconds
int StationListInterval = 600;   // in seconds
int SDES_Interval       = 10;    // in seconds
int MaxConferenceClients= 8;
int SF_Port             = 2074;
int SF_Enable           = 0;
int LinkConferences     = 1;
int SF_SWL_Only         = 0;
int SB_Enable           = 0;
int UserCountEnable     = 1;
int MaxCountEnable      = 1;
int UserCountInLocation = 1;
int PauseTime           = 0;
int iLinkDirServer      = 0;
int PrivateConference   = 0;
int EnableDiskCommands  = 1;
int DnsUpdateInterval   = 15 * 60;
int BelchTime           = 0;
int InactiveDirTimeout  = 60 * 60 * 24;   // one day

extern int LogFileRolloverType;

int MaxPlayBackPause    = 5;
int MinPlayBackPause    = 0;
int AudioTestConf       = 0;
int MaxPlayWithoutPause = (10 * 60);   // 10 minutes
int SaveInfoFiles       = 0;
int ShowStationInfo     = 0;
int WriteHostFile       = 0;
int AutoLurkTimeout     = (15 * 60);   // 15 minutes
int DefaultAutoLurk     = 0;
int BlabOffTimer        = 0;
int LogLevel            = 0;
int CmdPort             = 0;
int IdleTimeout         = 3 * 60;
int SF_ReplyPort        = 2074;
int DirCompression      = 1;
int WelcomeDelay        = 0;
int MaxDups             = 100;

int AddBannerLine(struct config_entry *p, char *Arg);
int SetWorkingDir(struct config_entry *p, char *Arg);
int SetConferenceCall(struct config_entry *p, char *Arg);
int SetQth(struct config_entry *p, char *Arg);
int CheckEmailAdr(struct config_entry *p, char *Arg);
int StartupCommand(struct config_entry *p, char *Arg);
int LoadHelpFile(struct config_entry *p, char *Arg);

// Pass 1 variables: processed before initialization
// Pass 2 variables: processed immediately after initialization
// Pass 3 variables: processed after first station list has been downloaded
// Pass 4 variables: processed after second successful directory server login
//                   (approximately one minute after pass 3)

#define NUM_CONFIG_PASSES  4

struct config_entry ConfigVars[] = {
// Required parameters
   { "ConferenceCall", "%f", &ConferenceCall, CON_FLG_REQUIRED, SetConferenceCall},
   { "ConferencePass", "%s", &ConferencePass, CON_FLG_REQUIRED},
   { "ConferenceQth", "%s", &ConferenceQth, CON_FLG_REQUIRED},
   { "WorkingDir", "%f", &WorkingDir, CON_FLG_REQUIRED, SetWorkingDir},
   { "EmailAdr", "%f", &EmailAdr, CON_FLG_REQUIRED, CheckEmailAdr },
// Optional parameters
   { "ConferenceID", "%s", &ConferenceID, 0},
   { "DirServerHost", "%s", &DirServerHost[0], 0},
   { "DirServerHost1", "%s", &DirServerHost[1], 0},
   { "DirServerHost2", "%s", &DirServerHost[2], 0},
   { "DirServerHost3", "%s", &DirServerHost[3], 0},
   { "RunAsUser", "%s", &RunAsUser, 0},
   { "AdminPass", "%s", &AdminPass, 0},
   { "SysopPass", "%s", &SysopPass, 0},
   { "RTP_Pass", "%s", &RTP_Pass, 0},
   { "WelcomeFile", "%s", &WelcomeFile, 0},
   { "Bind2IP", "%s", &Bind2IP, 0},
   { "AppName", "%s", &AppName, 0},
   { "RunDir", "%s", &RunDir, 0},
   { "Banner", "%f", &Banner, 0, AddBannerLine},
   { "ConfMemberTimeout", "%d", &ConfMemberTimeout, 0},
   { "ConfAudioTimeout", "%d", &ConfAudioTimeout, 0},
   { "LoginInterval", "%d", &LoginInterval, 0},
   { "StationListInterval", "%d", &StationListInterval, 0},
   { "SDES_Interval", "%d", &SDES_Interval, 0},
   { "MaxConferenceClients", "%d", &MaxConferenceClients, 0},
   { "SF_Port", "%d", &SF_Port, 0},
   { "SF_Enable", "%d", &SF_Enable, 0},
   { "SF_SWL_Only", "%d", &SF_SWL_Only, 0},
   { "SB_Enable", "%d", &SB_Enable, 0},
   { "UserCountEnable", "%d", &UserCountEnable, 0},
   { "MaxCountEnable", "%d", &MaxCountEnable, 0},
   { "UserCountInLocation", "%d", &UserCountInLocation, 0},
   { "LinkConferences", "%d", &LinkConferences, 0},
   { "PauseTime", "%d", &PauseTime, 0},
   { "iLinkDirServer", "%d", &iLinkDirServer, 0},
   { "PrivateConference", "%d", &PrivateConference, 0},
   { "EnableDiskCommands", "%d", &EnableDiskCommands, 0},
   { "DnsUpdateInterval", "%d", &DnsUpdateInterval, 0},
   { "BelchTime", "%d", &BelchTime, 0},
   { "InactiveDirTimeout", "%d", &InactiveDirTimeout, 0},
   { "LogFileRolloverType", "%d", &LogFileRolloverType, 0},
   { "MaxPlayBackPause", "%d", &MaxPlayBackPause, 0},
   { "MinPlayBackPause", "%d", &MinPlayBackPause, 0},
   { "AudioTestConf", "%d", &AudioTestConf, 0},
   { "MaxPlayWithoutPause", "%d", &MaxPlayWithoutPause, 0},
   { "SaveInfoFiles", "%d", &SaveInfoFiles, 0},
   { "ShowStationInfo", "%d", &ShowStationInfo, 0},
   { "WriteHostFile", "%d", &WriteHostFile, 0},
   { "AutoLurkTimeout", "%d", &AutoLurkTimeout, 0},
   { "DefaultAutoLurk", "%d", &DefaultAutoLurk, 0},
   { "BlabOffTimer", "%d", &BlabOffTimer, 0},
   { "LogLevel", "%d", &LogLevel, 0},
   { "CmdPort", "%d", &CmdPort, 0},
   { "IdleTimeout", "%d", &IdleTimeout, 0},
   { "SF_ReplyPort", "%d", &SF_ReplyPort, 0},
   { "DirCompression", "%d", &DirCompression, 0},
   { "EventScript", "%s", &EventScript, 0},
   { "StartupCmd", "%f", NULL, CON_FLG_PASS4, StartupCommand},
   { "HelpFile", "%f", NULL, 0, LoadHelpFile},
   { "WelcomeDelay", "%d", &WelcomeDelay, 0},
   { "MaxDups", "%d", &MaxDups, 0},
   { "" }   /* end of table marker */
};

void SetConfigDefaults(int bSet)
{
   int i;

   for(i = 0; ConfigVars[i].var_name[0] != 0; i++) {
      if(strcmp(ConfigVars[i].var_type,"%s") == 0 && 
         *((char **) ConfigVars[i].var_ptr) != NULL)
      { 
         free(*((char **) ConfigVars[i].var_ptr));
         *((char **) ConfigVars[i].var_ptr) = NULL;
      }
      else if(strcmp(ConfigVars[i].var_type,"%f") == 0) {
         ConfigVars[i].SetFunc(&ConfigVars[i],NULL);
      }
   }

   if(bSet) {
      ConferenceID = strdup(PACKAGE " V " VERSION);
      DirServerHost[0] = strdup("server1.echolink.org");
      AppName = strdup("tbd");
      RunDir = strdup("/var/run/");
   }
}

int AddBannerLine(struct config_entry *p, char *Arg)
{
   char *Temp;

   if(Arg != NULL) {
      if(Banner != NULL) {
      // Banner already has line(s)
         Temp = Banner;
         Banner = malloc(strlen(Temp)+strlen(Arg)+2);
         if(Banner != NULL) {
            strcpy(Banner,Temp);
            strcat(Banner,"\r");
            strcat(Banner,Arg);
         }
         free(Temp);
      }
      else {
      // First banner line
         Banner = strdup(Arg);
      }
   }
   else if(Banner != NULL) {
      free(Banner);
      Banner = NULL;
   }

   return 0;
}

int SetConferenceCall(struct config_entry *p, char *Arg)
{
   int Ret = 0;

   if(Arg != NULL) {
      if(ConferenceCall != NULL) {
         free(ConferenceCall);
      }

      ConferenceCall = strdup(Arg);

      if(strcmp(ConferenceCall,"W1AW") == 0) {
         LOG_ERROR(("Come on now, your call isn't really W1AW is it!\n"));
         LOG_ERROR(("Please read the documentation, edit tbd.conf and try again.\n"));
         Ret = ERR_CONFIG_FILE;
      }
   }
   else if(ConferenceCall != NULL) {
      free(ConferenceCall);
      ConferenceCall = NULL;
   }
   p->Flags |= CON_FLG_SET;
   return Ret;
}

int SetQth(struct config_entry *p, char *Arg)
{
   int Ret = 0;

   if(Arg != NULL) {
      if(ConferenceQth!= NULL) {
         free(ConferenceQth);
      }

      ConferenceQth = strdup(Arg);

      if(strcmp(ConferenceQth,"Newington, Ct.") == 0) {
         LOG_ERROR(("Come on now, you don't live in \"Newington, Ct.\" do you!\n"));
         LOG_ERROR(("Please read the documentation, edit tbd.conf and try again.\n"));
         Ret = ERR_CONFIG_FILE;
      }
   }
   else if(ConferenceQth != NULL) {
      free(ConferenceQth);
      ConferenceQth = NULL;
   }
   p->Flags |= CON_FLG_SET;
   return Ret;
}

int SetWorkingDir(struct config_entry *p, char *Arg)
{
   int Ret = 0;

   if(Arg != NULL) {
      if(WorkingDir != NULL) {
         free(WorkingDir);
      }
      WorkingDir = strdup(Arg);
      if(WorkingDir != NULL) {
         if(chdir(WorkingDir)) {
            LOG_ERROR(("Unable to change to working directory \"%s\", %s",
                       WorkingDir,Err2String(errno)));
            free(WorkingDir);
            WorkingDir = NULL;
            Ret = ERR_WORKING_DIR;
         }
      }
   }
   else if(WorkingDir != NULL) {
      free(WorkingDir);
      WorkingDir = NULL;
   }
   p->Flags |= CON_FLG_SET;
   return Ret;
}

int CheckEmailAdr(struct config_entry *p, char *Arg)
{
   int Ret = 0;

   if(Arg != NULL) {
      if(EmailAdr != NULL) {
         free(EmailAdr);
      }

      EmailAdr = strdup(Arg);

      if(strcmp(EmailAdr,"w1aw@arrl.net") == 0) {
         LOG_ERROR(("Come on now, your email address isn't \"w1aw@arrl.net\" "
                    "is it?\n"));
         Ret = ERR_CONFIG_FILE;
      }
      else if(!strchr(EmailAdr,'@') || !strchr(EmailAdr,'.')) {
         LOG_ERROR(("Error: Invalid email address \"%s\".\n"));
         Ret = ERR_CONFIG_FILE;
      }

      if(Ret != 0) {
         LOG_ERROR(("The email address is optional, please either delete the "
                    "entry\n"));
         LOG_ERROR(("or provide a valid email address.\n"));
      }
   }
   else if(ConferenceQth != NULL) {
      free(ConferenceQth);
      ConferenceQth = NULL;
   }
   p->Flags |= CON_FLG_SET;
   return Ret;
}

int StartupCommand(struct config_entry *p, char *Arg)
{
   if(Arg != NULL) {
      StartCmdHandler(Arg);
   }
   return 0;
}

int LoadHelpFile(struct config_entry *p, char *Arg)
{
   struct stat FileStats;
   FILE *fp;
   char *cp;
   int Ret = 0;

   if(CustomHelp != NULL) {
      free(CustomHelp);
      CustomHelp = NULL;
   }

   if(Arg != NULL) {
      if(stat(Arg,&FileStats) != 0 || (fp = fopen(Arg,"r")) == NULL) {
         LOG_ERROR(("Error: unable to open help file \"%s\" %s",Arg,
                    Err2String(errno)));
         Ret = ERR_CONFIG_FILE;
      }
      else if((CustomHelp = malloc(FileStats.st_size)) != NULL) {
         if(fread(CustomHelp,FileStats.st_size,1,fp) != 1) {
            LOG_ERROR(("Error reading help file \"%s\", %s",Arg,
                       Err2String(errno)));
            free(CustomHelp);
            CustomHelp = NULL;
            Ret = ERR_CONFIG_FILE;
         }
         else {
         // Convert the '\n' end of line convention to '\r' used by EchoLink
            cp = CustomHelp;
            while((cp = strchr(cp,'\n')) != NULL) {
               *cp++ = '\r';
            }
         }
         fclose(fp);
      }
   }
   return Ret;
}

int DoConfigPass(int Pass)
{
   char *ConfigFile = ConfigFilePath;
   static FILE *fp = NULL;
   int Ret = 0;
    
   ConfigPass = Pass;
   if(ConfigFile == NULL) {
      ConfigFile = CONFIG_FILE_PATH;
   }
   
   if(Pass == 1 && (fp = fopen(ConfigFile,"r")) == NULL) {
      LOG_ERROR(("Fatal error opening configuration file, \"%s\".\n",
                 ConfigFile));
      Ret = CONFIG_OPEN_ERROR;
   }
   else {
      fseek(fp,0,SEEK_SET);
      Ret = read_config(fp,ConfigVars,Pass);
      if(Pass == NUM_CONFIG_PASSES) {
         fclose(fp);
         fp = NULL;
      }
   }

   return Ret;
}


