/* Copyright (C) 2003 GFRN systems

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
   See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   The latest version of this program may be found at
   http://CQiNet.sourceforge.net

   $Log: tbdcmd.c,v $
   Revision 1.1  2003/08/16 14:27:27  wb6ymh
   Initial import: command line utility to send commands to thebridge.


*/

#include "common.h"

#ifndef _WIN32
   // FreeBSD, Linux, etc..
   #include <stdio.h>
   #ifdef HAVE_UNISTD_H
      #include <unistd.h>
   #endif
   
   #ifdef STDC_HEADERS
      #include <stdlib.h>
   #endif
   
   #ifdef TIME_WITH_SYS_TIME
      #include <sys/time.h>
      #include <time.h>
   #else
      #ifdef HAVE_SYS_TIME_H
         #include <sys/time.h>
      #else
         #include <time.h>
      #endif
   #endif
   #include <errno.h>
   
   #ifdef HAVE_STRINGS_H
      #include <string.h>
   #endif
   #include <assert.h>
   #include <sys/socket.h>
   #include <netinet/in.h>
   #include <sys/resource.h>
   #include <arpa/inet.h>
   #include <string.h>
#else
   // Windoze
   #include <stdio.h>
   #include <winsock2.h>
   #include <string.h>
#endif

#include "ilink.h"

#ifndef  FALSE
#define  FALSE    0
#endif

#ifndef  TRUE
#define  TRUE  !FALSE
#endif

typedef union {
   struct sockaddr s;
   struct sockaddr_in i;
   #define ADDR   i.sin_addr.s_addr
   #define PORT   i.sin_port
} IPAdrUnion;


#ifdef _WIN32
// Windoze
   WSADATA wsad;
#endif

int bQuiet = 0;
int bSlient = 0;

#ifndef HAVE_GETOPT
// globals needed by getopt
int optind = 1;
int optopt;
char *optarg;
int getopt(int argc,char **argv,char *opts);
#endif

void Usage()
{
   printf("Usage: tbdcmd [-p<port>] [-q] [-s] [commands] \n");
   printf("\t-p - Specify cmdline port (default 5198).\n");
   printf("\t-q - Quiet, numeric result code only.\n");
   printf("\t-s - Slient, suppress banner.\n");
}

int main(int argc, char* argv[])
{
   int Ret = 0;
   int Option;
   int MySocket;
   IPAdrUnion HisAdr;
   char  Line[1024];
   char  *cp;
   int   BytesRxed;
   int   BytesSent;
   int   FirstArg = 1;
   int   ArgCount = argc;
   int   WritePoint;
   int   Wrote;
   int   i;
   int   Port = ILINK_RTP_PORT;

#ifndef _WIN32
   struct rlimit rlim;

// On some systems core dumps are disabled by default, 
// enable them so we can debug this thing !

   rlim.rlim_cur = RLIM_INFINITY;
   if(setrlimit(RLIMIT_CORE, &rlim)) {
      perror("Unable to enable code dumps: ");
   }
#else
   /* Initialize Winsock*/
   if(WSAStartup(0x0101, &wsad) != 0) {
      printf("Fatal error: unable to initialize Winsock\n");
      exit(3);
   }
#endif

   while((Option = getopt(argc, argv, "p:qs")) != -1 && !Ret) {
      FirstArg++;
      ArgCount--;
      switch(Option) {
         case 'p':   // port
            if(sscanf(optarg,"%d",&Port) != 1 || Port <= 0 || Port > 0xffff) {
               printf("tbdcmd: Error invalid port number \"%s\"\n",optarg);
               exit(3);
            }
            break;

         case 'q':   // quiet 
            bQuiet = TRUE;
            break;

         case 's':   // slient
            bSlient = TRUE;
            break;

         case '?':
            Usage();
      }
   }

   if(!bQuiet && !bSlient && !Ret) {
      printf("tbdcmd Version " VERSION " compiled "__DATE__  " " __TIME__"\n");
   }

   if(ArgCount > 1) {
      WritePoint = 0;
      for(i = 0; i < ArgCount -1 ; i++) {
         Wrote = snprintf(&Line[WritePoint],sizeof(Line)-WritePoint-1,"%s%s",
                          WritePoint == 0 ? "" : " ",argv[FirstArg+i]);
         if(Wrote > 0 && WritePoint + Wrote < sizeof(Line)) {
            WritePoint += Wrote;
         }
         else {
         // Buffer overflow
            break;
         }
      }
      Line[WritePoint] = 0;
   }


   if((MySocket = socket(AF_INET,SOCK_DGRAM,0)) == SOCKET_ERROR) {
      printf("Socket() failed.\n");
   }
   else {
      HisAdr.i.sin_family = AF_INET;
      HisAdr.PORT = htons((u_short) Port);
      HisAdr.ADDR = inet_addr("127.0.0.1");

      do {
         if(ArgCount == 1) {
         // No arguments enter interactive mode
            printf("tbd> ");
            if(fgets(Line,sizeof(Line),stdin) == NULL) {
               break;
            }

            if((cp = strchr(Line,'\n')) != NULL) {
               *cp = 0;
            }

            if(strlen(Line) == 0) {
            // exit on empty line
               break;
            }
         }
         
         cp = Line;
         if(*cp == '.') {
         // Skip leading '.' if entered, it's not needed here.
            cp++;
         }
         
         BytesSent = sendto(MySocket,cp,strlen(cp)+1,0,&HisAdr.s,sizeof(HisAdr));

         if(BytesSent == SOCKET_ERROR) {
            Ret = ERROR_CODE;
            printf("Error: sendto() failed (%d)\n",Ret);
            break;
         }

         BytesRxed = recv(MySocket,Line,sizeof(Line)-1,0);
         if(BytesRxed == SOCKET_ERROR) {
            Ret = ERROR_CODE;
            printf("Error: recv() failed (%d)\n",Ret);
            break;
         }
         
         Line[BytesRxed] = 0;
         
         Ret = atoi(Line);

         if(bQuiet) {
            if((cp = strchr(Line,'\n')) != NULL) {
            // Only want a single line, truncate the output
               cp[1] = 0;
            }
         }

         printf("%s",Line);
      } while(ArgCount == 1);
   }
   
   return Ret;
}

#ifndef HAVE_GETOPT

/*
 * getopt a wonderful little function that handles the command line.
 * available courtesy of AT&T.
 */
int getopt(int argc,char **argv,char *opts)
{
   static int sp = 1;
   register int c;
   register char *cp;

   if(sp == 1)
      if(optind >= argc ||
         argv[optind][0] != '-' || argv[optind][1] == '\0')
         return(EOF);
      else if(strcmp(argv[optind], "--") == 0) {
         optind++;
         return(EOF);
      }
   optopt = c = argv[optind][sp];
   if(c == ':' || (cp=strchr(opts, c)) == NULL) {
      printf("%s: illegal option -- ", argv[0],c);
      if(argv[optind][++sp] == '\0') {
         optind++;
         sp = 1;
      }
      return('?');
   }
   if(*++cp == ':') {
      if(argv[optind][sp+1] != '\0')
         optarg = &argv[optind++][sp+1];
      else if(++optind >= argc) {
         printf("%s: option requires an argument -- ", argv[0],c);
         sp = 1;
         return('?');
      }
      else
         optarg = argv[optind++];
      sp = 1;
   }
   else {
      if(argv[optind][++sp] == '\0') {
         sp = 1;
         optind++;
      }
      optarg = NULL;
   }
   return(c);
}
#endif
